#ifdef GL_ES
precision highp float;
#endif

uniform sampler2D texture;

uniform float time;
uniform float speed;
uniform float frequency;
uniform float amplitude;
uniform float granularity;
uniform vec2  bounds;

varying vec4 vertexColor;
varying vec2 texCoord;

vec2 shift( vec2 p ) {
    float d = time * speed;
    vec2 f = frequency * (p + d);
    vec2 q = cos( vec2(
       cos(f.x-f.y)*cos(f.y),
       sin(f.x+f.y)*sin(f.y) ) );
    return q;
}

vec2 hash2(vec2 p ) {
   return fract(sin(vec2(dot(p, vec2(123.4, 748.6)), dot(p, vec2(547.3, 659.3))))*5232.85324);
}
float hash(vec2 p) {
  return fract(sin(dot(p, vec2(43.232, 75.876)))*4526.3257);
}

//Based off of iq's described here: http://www.iquilezles.org/www/articles/voronoilin
float voronoi(vec2 p) {
    vec2 n = floor(p);
    vec2 f = fract(p);
    float md = 5.0;
    vec2 m = vec2(0.0);
    for (int i = -1;i<=1;i++) {
        for (int j = -1;j<=1;j++) {
            vec2 g = vec2(i, j);
            vec2 o = hash2(n+g);
            o = 0.5+0.5*sin(time+5.038*o);
            vec2 r = g + o - f;
            float d = dot(r, r);
            if (d<md) {
              md = d;
              m = n+g+o;
            }
        }
    }
    return md;
}

float ov(vec2 p) {
    float v = 0.0;
    float a = 0.4;
    for (int i = 0;i<3;i++) {
        v+= voronoi(p)*a;
        p*=2.0;
        a*=0.5;
    }
    return v;
}

void main()
{
    vec2 r = texCoord.xy;
    vec2 p = shift(r);
    vec2 q = shift(r + 1.0);
    vec2 s = r + amplitude * (p - q);

    vec3 a = vec3(0.0, 0.0, 0.0);
    vec3 b = vec3(1.0, 1.0, 1.0);

    vec4 i = vec4(mix(a, b, smoothstep(bounds.x, bounds.y, ov(r * granularity))).rgb, 0);

	gl_FragColor = texture2D(texture, s) * (vertexColor + i);
}
